`line 1 "F:\verilog\divider\dividers\bench\verilog\bench_div_top.v"





























































`line 1 "F:\verilog\divider\dividers\bench\verilog\timescale.v"
`timescale 1ns / 10ps


`line 62 "F:\verilog\divider\dividers\bench\verilog\bench_div_top.v"


module bench_div_top();

	parameter z_width = 16;
	parameter d_width = z_width /2;

	parameter pipeline = d_width +4;

	parameter show_div0 = 0;
	parameter show_ovf  = 0;

	
	
	
	reg clk;

	integer z, d, n;
	integer dz [pipeline:1];
	integer dd [pipeline:1];
	reg [d_width:1] di;
	reg [z_width:1] zi;

	integer sr, qr;

	wire [d_width   :0] s;
	wire [d_width   :0] q;
	wire div0, ovf;
	reg  [d_width :0] sc, qc;

	integer err_cnt;

	function integer twos;
		input [d_width:1] d;
	begin
	  if(d[d_width])
	    twos = -(~d[d_width:1] +1);
	  else
	    twos = d[d_width:1];
	end
	endfunction

	
	
	
	div_su #(z_width) dut (
		.clk(clk),
		.ena(1'b1),
		.z(zi),
		.d(di),
		.q(q),
		.s(s),
		.div0(div0),
		.ovf(ovf)
	);

	always #2.5 clk <= ~clk;


	always @(posedge clk)
	  for(n=2; n<=pipeline; n=n+1)
	     begin
	         dz[n] <= #1 dz[n-1];
	         dd[n] <= #1 dd[n-1];
	     end

	initial
	begin
	    $display("*");
	    $display("* Starting testbench");
	    $display("*");







	    err_cnt = 0;

	    clk = 0; 

	    
	    @(posedge clk);

	    
	    for(z=-(1<<(z_width -1)); z < 1<<(z_width -1); z=z+1)
	    for(d=0; d< 1<<(z_width/2); d=d+1)
	    begin
	        zi <= #1 z;
	        di <= #1 d;

	        dz[1] <= #1 z;
	        dd[1] <= #1 d;
	
	
	
	 
	  
	        qc = dz[pipeline] / dd[pipeline];
	        sc =dz[pipeline] - (dd[pipeline] * (dz[pipeline]/dd[pipeline]));

	        if(!ovf && !div0)
	          if ( (qc !== q) || (sc !== s) )
	            begin
	                $display("Result error (z/d=%0d/%0d). Received (q,s) = (%0d,%0d), expected (%0d,%0d)",
	                         dz[pipeline], dd[pipeline], twos(q), s, twos(qc), sc);

	                err_cnt = err_cnt +1;
	            end

	          if(show_div0)
	            if(div0)
	              $display("Division by zero (z/d=%0d/%0d)", dz[pipeline], dd[pipeline]);

	          if(show_ovf)
	            if(ovf)
	              $display("Overflow (z/d=%0d/%0d)", dz[pipeline], dd[pipeline]);

	          @(posedge clk);
	    end

	    
	    repeat(20) @(posedge clk);

	    $display("*");
	    $display("* Testbench ended. Total errors = %d", err_cnt);
	    $display("*");

	    $stop;
	end

endmodule


`line 1 "F:\verilog\divider\dividers\bench\verilog\timescale.v"
`timescale 1ns / 10ps



`line 1 "F:\verilog\divider\dividers\rtl\verilog\div.v"



















































`line 1 "F:\verilog\divider\dividers\bench\verilog\timescale.v"
`timescale 1ns / 10ps


`line 52 "F:\verilog\divider\dividers\rtl\verilog\div.v"


module div(clk, ena, z, d, q, s);

	
	
	
	parameter z_width = 16;
	parameter d_width = z_width /2;

	
	
	
	input clk;               
	input ena;               

	input  [z_width -1:0] z; 
	input  [d_width -1:0] d; 
	output [z_width -1:0] q; 
	output [z_width -1:0] s; 

	
	
	
	function sc;
	  input [z_width:0] si;
	  input [d_width:0] di;
	begin
	    sc = si[z_width] ~^ di[d_width];
	end
	endfunction

	function [z_width:0] gen_q;
	  input [z_width:0] q;
	  input             q0;
	begin
	    gen_q = {(q << 1), q0};
	end
	endfunction

	function [z_width:0] gen_s;
	  input [z_width:0] si;
	  input [z_width:0] di;
	  input             sel;
	begin
	    if(sel)
	      gen_s = {si[z_width-1:0], 1'b0} - di;
	    else
	      gen_s = {si[z_width-1:0], 1'b0} + di;
	end
	endfunction

	
	
	
	reg [z_width:0] q_pipe  [z_width:0];
	reg [z_width:0] s_pipe  [z_width:0];
	reg [z_width:0] d_pipe  [z_width:0];
	reg [z_width:0] qb_pipe;

	
	
	
	
	initial
	begin
	  if(d_width > z_width)
	    $display("div.v parameter error (d_width > z_width). Divisor width larger than divident width.");
	end
	

	integer n;

	
	always @(d)
	  d_pipe[0] <= {d[d_width -1], d, {(z_width-d_width){1'b0}} };

	always @(posedge clk)
	  if(ena)
	    for(n=1; n < z_width; n=n+1)
	       d_pipe[n] <= #1 d_pipe[n-1];

	
	always
	  begin
	    #1;
	    for(n=0; n < z_width; n=n+1)
	       qb_pipe[n] <= sc(s_pipe[n], d_pipe[n]);
	  end

	
	always@(z)
		s_pipe[0] <= {z[z_width -1], z};

	always @(posedge clk)
	  if(ena)
	    for(n=1; n < z_width; n=n+1)
	       s_pipe[n] <= #1 gen_s(s_pipe[n-1], d_pipe[n-1], qb_pipe[n-1]);

	
	always @(qb_pipe[0])
	  q_pipe[0] <= #1 { {(z_width){1'b0}}, qb_pipe[0]};

	always @(posedge clk)
	  if(ena)
	    for(n=1; n < z_width; n=n+1)
	       q_pipe[n] <= #1 {q_pipe[n-1], qb_pipe[n]};

	wire [z_width:0] last_q;
	assign last_q = q_pipe[z_width -1];

	always @(posedge clk)
	  if(ena)
	    q_pipe[z_width] <= #1 {!last_q[z_width-1], last_q[z_width-2:0], 1'b1};

	
	assign q = q_pipe[z_width];
	assign s = s_pipe[z_width];
endmodule




`line 1 "F:\verilog\divider\dividers\rtl\verilog\div_su.v"
























































`line 1 "F:\verilog\divider\dividers\bench\verilog\timescale.v"
`timescale 1ns / 10ps


`line 57 "F:\verilog\divider\dividers\rtl\verilog\div_su.v"



module div_su(clk, ena, z, d, q, s, div0, ovf);

	
	
	
	parameter z_width = 16;
	parameter d_width = z_width /2;
	
	
	
	
	input clk;              
	input ena;              

	input  [z_width-1:0] z; 
	input  [d_width-1:0] d; 
	output [d_width  :0] q; 
	output [d_width  :0] s; 
	output div0;
	output ovf;

	reg [d_width:0] q, s;
	reg div0;
	reg ovf;

	
	
	
	reg [z_width -1:0] iz;
	reg [d_width -1:0] id;
	reg [d_width +1:0] spipe;

	wire [d_width -1:0] iq, is;
	wire                idiv0, iovf;

	
	
	

	
	always @(posedge clk)
	  if (ena)
	    id <= #1 d;

	
	always @(posedge clk)
	  if (ena)
	    if (z[z_width-1])
	       iz <= #1 ~z +1'h1;
	    else
	       iz <= #1 z;

	
	integer n;
	always @(posedge clk)
	  if(ena)
	  begin
	      spipe[0] <= #1 z[z_width-1];

	      for(n=1; n <= d_width+1; n=n+1)
	         spipe[n] <= #1 spipe[n-1];
	  end

	
	div_uu #(z_width, d_width)
	divider (
		.clk(clk),
		.ena(ena),
		.z(iz),
		.d(id),
		.q(iq),
		.s(is),
		.div0(idiv0),
		.ovf(iovf)
	);

	
	always @(posedge clk)
	  if(ena)
	    if(spipe[d_width+1])
	    begin
	        q <= #1 (~iq) + 1'h1;
	        s <= #1 (~is) + 1'h1;
	    end
	    else
	    begin
	        q <= #1 {1'b0, iq};
	        s <= #1 {1'b0, is};
	    end

	
	always @(posedge clk)
	  if(ena)
	  begin
	      div0 <= #1 idiv0;
	      ovf  <= #1 iovf;
	  end
endmodule


`line 1 "F:\verilog\divider\dividers\rtl\verilog\div_uu.v"



























































`line 1 "F:\verilog\divider\dividers\bench\verilog\timescale.v"
`timescale 1ns / 10ps


`line 60 "F:\verilog\divider\dividers\rtl\verilog\div_uu.v"



module div_uu(clk, ena, z, d, q, s, div0, ovf);

	
	
	
	parameter z_width = 16;
	parameter d_width = z_width /2;
	
	
	
	
	input clk;               
	input ena;               

	input  [z_width -1:0] z; 
	input  [d_width -1:0] d; 
	output [d_width -1:0] q; 
	output [d_width -1:0] s; 
	output div0;
	output ovf;
	reg [d_width-1:0] q;
	reg [d_width-1:0] s;
	reg div0;
	reg ovf;

	
	
	
	function [z_width:0] gen_s;
		input [z_width:0] si;
		input [z_width:0] di;
	begin
	  if(si[z_width])
	    gen_s = {si[z_width-1:0], 1'b0} + di;
	  else
	    gen_s = {si[z_width-1:0], 1'b0} - di;
	end
	endfunction

	function [d_width-1:0] gen_q;
		input [d_width-1:0] qi;
		input [z_width:0] si;
	begin
	  gen_q = {qi[d_width-2:0], ~si[z_width]};
	end
	endfunction

	function [d_width-1:0] assign_s;
		input [z_width:0] si;
		input [z_width:0] di;
		reg [z_width:0] tmp;
	begin
	  if(si[z_width])
	    tmp = si + di;
	  else
	    tmp = si;

	  assign_s = tmp[z_width-1:z_width-d_width];
	end
	endfunction

	
	
	
	reg [d_width-1:0] q_pipe  [d_width-1:0];
	reg [z_width:0] s_pipe  [d_width:0];
	reg [z_width:0] d_pipe  [d_width:0];

	reg [d_width:0] div0_pipe, ovf_pipe;
	
	
	
	
	initial
	begin
	  if(d_width !== z_width / 2)
	    $display("div.v parameter error (d_width != z_width/2).");
	end
	

	integer n0, n1, n2, n3;

	
	always @(d)
	  d_pipe[0] <= {1'b0, d, {(z_width-d_width){1'b0}} };

	always @(posedge clk)
	  if(ena)
	    for(n0=1; n0 <= d_width; n0=n0+1)
	       d_pipe[n0] <= #1 d_pipe[n0-1];

	
	always @(z)
	  s_pipe[0] <= z;

	always @(posedge clk)
	  if(ena)
	    for(n1=1; n1 <= d_width; n1=n1+1)
	       s_pipe[n1] <= #1 gen_s(s_pipe[n1-1], d_pipe[n1-1]);

	
	always @(posedge clk)
	  q_pipe[0] <= #1 0;

	always @(posedge clk)
	  if(ena)
	    for(n2=1; n2 < d_width; n2=n2+1)
	       q_pipe[n2] <= #1 gen_q(q_pipe[n2-1], s_pipe[n2]);


	
	always @(z or d)
	begin
	  ovf_pipe[0]  <= !(z[z_width-1:d_width] < d);
	  div0_pipe[0] <= ~|d;
	end

	always @(posedge clk)
	  if(ena)
	    for(n3=1; n3 <= d_width; n3=n3+1)
	    begin
	        ovf_pipe[n3] <= #1 ovf_pipe[n3-1];
	        div0_pipe[n3] <= #1 div0_pipe[n3-1];
	    end

	
	always @(posedge clk)
	  if(ena)
	    ovf <= #1 ovf_pipe[d_width];

	always @(posedge clk)
	  if(ena)
	    div0 <= #1 div0_pipe[d_width];

	always @(posedge clk)
	  if(ena)
	    q <= #1 gen_q(q_pipe[d_width-1], s_pipe[d_width]);

	always @(posedge clk)
	  if(ena)
	    s <= #1 assign_s(s_pipe[d_width], d_pipe[d_width]);
endmodule






